/*
Module : CPUTICKER.CPP
Purpose: Implementation for a high resolution counter class. The class wraps the Pentium-specific 
         time stamp counter available via the RDTSC assembly language instruction. This counter has 
         a resolution in terms of PCLKS (processor clocks) so it can be used for direct instruction 
         timings. Please note that you should be aware of various issues associated with using the
         RDTSC instruction for timing purposes. For further information on the issues please see 
         the MSDN article at http://msdn.microsoft.com/library/default.asp?url=/library/en-us/directx9_c/Game_Timing_and_Multicore_Processors.asp

Copyright (c) 1996 - 2006 by J.M.McGuiness (www.hussar.demon.co.uk)
                             PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/

//////////////////////////// Macros / Defines /////////////////////////////////

#pragma once

#ifndef __CPUTICKER_H__
#define __CPUTICKER_H__

#ifndef CCPUTICKER_EXT_CLASS
#define CCPUTICKER_EXT_CLASS
#endif


////////////////////////// Classes ////////////////////////////////////////////

class CCPUTICKER_EXT_CLASS CCPUTicker
{
public:
//perform the actual measurement
  unsigned __int64 Measure();

//static methods
	static BOOL GetCPUFrequency(double& CPUFrequency, double& TargetAverageDeviation, DWORD dwInterval = 100, DWORD dwMaxLoops = 20);
  static BOOL GetCachedCPUFrequency(double& CPUFrequency, double& AverageDeviation);
  static double GetTickCountAsSeconds(unsigned __int64 nValue);  

protected:
//Static Member variables
  static double m_dAverageDeviation;
  static double m_dCPUFrequency;
  static BOOL   m_bFrequencyCalculated;
};

#endif //__CPUTICKER_H__
