/***********************************************************
*  File:     RealGoodTimingDlg.cpp
*
*  Author:   Michael Kennedy
*  Email:    mkennedy@math.ucsd.edu
*  WWW:      http://math.ucsd.edu/~mkennedy/software
*
*  Date:     Created on 7/12/00
*
*  Purpose:  To demonstrate the CCPUTicker Class by Naughter 
*            and McGuiness - See CPUTicker.h for details.
*
*************************************************************/

#include "stdafx.h"
#include "RealGoodTiming.h"
#include "RealGoodTimingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CAboutDlg)
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CRealGoodTimingDlg::CRealGoodTimingDlg(CWnd* pParent)	: CDialog(CRealGoodTimingDlg::IDD, pParent),
                                                        m_nStartTick(0),
                                                        m_bCounting(FALSE),
	                                                      m_bInitializedTimers(FALSE)
{
	//{{AFX_DATA_INIT(CRealGoodTimingDlg)
	m_DT = _T("NO ELAPSED TIME");
	m_sMessages = _T("Hit \"Initialize Timers\" to begin...");
	//}}AFX_DATA_INIT
	
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CRealGoodTimingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRealGoodTimingDlg)
	DDX_Text(pDX, IDC_STATIC_DT, m_DT);
	DDX_Text(pDX, IDC_STATIC_MESSAGES, m_sMessages);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRealGoodTimingDlg, CDialog)
	//{{AFX_MSG_MAP(CRealGoodTimingDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_STOP_COUNT, OnButtonStopCount)
	ON_BN_CLICKED(IDC_BUTTON2, OnButtonInitialize)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CRealGoodTimingDlg::OnInitDialog()
{
  //Let the base class do its thing
	CDialog::OnInitDialog();

	SetTimer(1,50,NULL);

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	return TRUE;
}

void CRealGoodTimingDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CRealGoodTimingDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRealGoodTimingDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CRealGoodTimingDlg::OnButtonStart() 
{  
	if (!m_bInitializedTimers) 
	{
		m_sMessages= _T("High resolution timers have not been initialized!");
		UpdateData(FALSE);
		return;
	}
	
	m_bCounting = TRUE;
  m_sMessages = _T("Counting... Hit stop when you're ready.");
	m_DT.Empty();
	UpdateData(FALSE);
	
	m_nStartTick = m_Counter.Measure();
}

void CRealGoodTimingDlg::OnButtonStopCount() 
{
	if (m_bCounting) 
	{	
  	unsigned __int64 nEndTick = m_Counter.Measure();
	  m_bCounting = FALSE;
		m_sMessages = _T("Elapsed time computed -- REAL ACCURATELY!");
		m_DT.Format(_T("%f seconds."), CCPUTicker::GetTickCountAsSeconds(nEndTick - m_nStartTick));
		UpdateData(FALSE);
	}
}

void CRealGoodTimingDlg::OnButtonInitialize()
{
	m_bInitializedTimers = TRUE;

  CWaitCursor wait;
	
	//Warn that program may look like it has hung while running
	m_sMessages = _T("This program will appear to hang for up to 2 seconds while determining CPU speed. This is normal behavior.");
	UpdateData(FALSE);

  double CPUFrequency;
  double TargetAverageDeviation;
	CCPUTicker::GetCPUFrequency(CPUFrequency, TargetAverageDeviation);
	
	m_sMessages = _T("The high perfomace timers are now ready for use.");
	UpdateData(FALSE);
}

void CRealGoodTimingDlg::OnTimer(UINT nIDEvent) 
{
	if (m_bCounting) 
	{
		unsigned __int64 nEndTick = m_Counter.Measure();
		m_DT.Format(_T("%f seconds."), CCPUTicker::GetTickCountAsSeconds(nEndTick - m_nStartTick));
		UpdateData(FALSE);
	}

  //let the base class do its thing
	CDialog::OnTimer(nIDEvent);
}
